import typer
from typing import Optional, List
from pathlib import Path

from ddot import lib
from devinstaller_core import exception as e
import anymarkup
import os.path
import os
from devinstaller_core import settings as s

app = typer.Typer()


DEFAULT_SPEC_FILES = ["devfile.toml", "devfile.yaml", "devfile.json"]
DEFAULT_SPEC_FILE = DEFAULT_SPEC_FILES[0]
DEFAULT_PROG_FILE = "devfile.py"


def get_spec_path(provided_path: Path) -> Path:
    if provided_path:
        return provided_path
    for i in DEFAULT_SPEC_FILES:
        if os.path.isfile(i):
            return Path(i)
    raise e.DevinstallerBaseException(
        "I wasn't able to find any spec file in the current directory, so you need to provide the path to a spec file."
    )


def goodbye() -> None:
    """Final callback function. Runs after the end of everything."""
    typer.secho("\nBye. Have a good day.", fg="green")


def get_path_relative_to_main(file_path: str):
    """Get the path relative to the script file
    """
    script_dir = os.path.dirname(__file__)  # <-- absolute dir the script is in
    full_path = os.path.join(script_dir, file_path)
    return Path(full_path).resolve()


def read_file(file_path: Path):
    """Read file
    """
    with open(str(file_path), "r") as _f:
        return _f.read()


def get_version():
    """Get the version using the `pyproject.toml` file
    """
    paths = ["../pyproject.toml", "pyproject.toml"]
    for p in paths:
        try:
            py_project_path = get_path_relative_to_main(p)
            data = read_file(py_project_path)
            data = anymarkup.parse(data, format="toml")
            return data["tool"]["poetry"]["version"]
        except Exception:
            pass
    return "N/A"


__version__ = get_version()


def version_callback(value: bool):
    """Callback function for version cli option
    """
    if value:
        typer.echo(f"Ddot Version: {__version__}")
        raise typer.Exit()


# @app.command()
# def install(
#     spec_file: Optional[Path] = typer.Option(DEFAULT_SPEC_FILE),
#     platform: Optional[str] = None,
#     module: Optional[str] = None,
# ) -> None:
#     """Install the default group and the modules which it requires"""
#     try:
#         req_list = [module] if isinstance(module, str) else None
#         d = lib.Devinstaller()
#         d.install(
#             spec_file_path=f"file: {spec_file}",
#             platform_codename=platform,
#             requirements_list=req_list,
#         )
#     except e.DevinstallerBaseException as err:
#         typer.secho(str(err), fg="red")


@app.command()
def show(spec_file: Optional[Path] = typer.Option(None)) -> None:
    """Show all the groups and modules available for your OS"""

    spec_file = get_spec_path(spec_file)
    d = lib.Devinstaller()
    d.show(spec_file_path=f"file: {spec_file}")


@app.command()
def run(
    module: Optional[List[str]] = typer.Option(None, "--module", "-m"),
    spec_file: Optional[Path] = typer.Option(None),
    verbose: bool = False,
):
    """Run commands"""

    s.settings.DDOT_VERBOSE = verbose
    spec_file = get_spec_path(spec_file)
    req_list = list(module)
    d = lib.Devinstaller()
    d.install(
        spec_file_path=f"file: {spec_file}",
        platform_codename=None,
        requirements_list=req_list,
    )


@app.callback()
def callback(
    ctx: typer.Context,
    _: Optional[bool] = typer.Option(
        None, "--version", callback=version_callback, is_eager=True
    ),
):
    pass


def main():
    try:
        app()
    except e.DevinstallerBaseException as err:
        typer.secho(str(err), fg="red")
    finally:
        goodbye()


if __name__ == "__main__":
    main()



from typing import Any, Dict, List, Optional

# from typeguard import typechecked
import typer

from devinstaller_core import lib
from devinstaller_core import dependency_graph as dg
from devinstaller_core import common_models as m


def get_req_list(
    dependency_graph: dg.DependencyGraph, requirements_list: List[str]
) -> List[str]:
    """Get the requirement list
    """
    list_of_deps = list(dependency_graph.graph.keys())
    if len(list_of_deps) == 1:
        return list_of_deps
    elif requirements_list is not None and requirements_list != []:
        return requirements_list
    else:
        return lib.get_requirement_list(dependency_graph.module_list())


class Devinstaller:
    def install(
        self,
        spec_file_path: Optional[str] = None,
        prog_file_path: Optional[str] = None,
        spec_object: Optional[Dict[Any, Any]] = None,
        platform_codename: Optional[str] = None,
        requirements_list: Optional[List[str]] = None,
    ) -> None:
        """Install the default preset and the modules which it requires."""
        validated_schema_object: m.TypeFullDocument = lib.core(
            file_path=spec_file_path, spec_object=spec_object
        )
        dependency_graph: dg.DependencyGraph = lib.create_dependency_graph(
            schema_object=validated_schema_object, platform_codename=platform_codename
        )
        req_list: List[str] = get_req_list(
            dependency_graph=dependency_graph, requirements_list=requirements_list
        )
        dependency_graph.install(req_list)
        orphan_modules_names = dependency_graph.orphan_modules
        if orphan_modules_names != set() and lib.get_user_confirmation(
            orphan_modules_names
        ):
            dependency_graph.uninstall_orphan_modules()

    def show(self, spec_file_path: Optional[str] = None,) -> None:
        """Install the default preset and the modules which it requires."""
        validated_schema_object: m.TypeFullDocument = lib.core(file_path=spec_file_path)
        dependency_graph: dg.DependencyGraph = lib.create_dependency_graph(
            schema_object=validated_schema_object, platform_codename=None
        )
        available_modules = list(dependency_graph.graph.keys())
        print("All the available modules\n")
        for i in available_modules:
            typer.secho(i, fg="cyan")

    # @typechecked
    # def run(
    #     self,
    #     interface_name: Optional[str] = None,
    #     spec_file_path: Optional[str] = None,
    #     prog_file_path: Optional[str] = None,
    #     spec_object: Optional[Dict[Any, Any]] = None,
    #     platform_codename: Optional[str] = None,
    # ) -> None:
    #     """The `run` function.

    #     This function is used for the interface block.
    #     """
    #     schema_object = self.hook.core(spec_file_path, spec_object)
    #     dev_module = self.hook.load_devfile(
    #         schema_object=schema_object, prog_file_path=prog_file_path
    #     )
    #     interface = m.get_interface(
    #         interface_list=schema_object["interfaces"], interface_name=interface_name
    #     )
    #     dependency_graph = self.hook.create_dependency_graph(
    #         schema_object=schema_object, platform_codename=platform_codename
    #     )



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['ddot']

package_data = \
{'': ['*']}

install_requires = \
['devinstaller-core>=0.9.1,<0.10.0', 'typer>=0.3.2,<0.4.0']

entry_points = \
{'console_scripts': ['ddot = ddot.__main__:main']}

setup_kwargs = {
    'name': 'ddot',
    'version': '0.4.0',
    'description': 'Dotfile manager using the Devinstaller framework',
    'long_description': '[![img](https://img.shields.io/badge/Made_in-VSCode-blue?style=for-the-badge)](https://code.visualstudio.com/download)\n[![img](https://img.shields.io/badge/follow_me-@alka1e-E4405F?style=for-the-badge&logo=instagram&labelColor=8f3c4c&logoColor=white)](https://www.instagram.com/alka1e)\n[![img](https://img.shields.io/badge/follow_me-@alka1e-1DA1F2?style=for-the-badge&logo=twitter&labelColor=27597a&logoColor=white)](https://twitter.com/alka1e)\n\n# Ddot\n\n_Your dev environment manager_\n\n[![img](https://img.shields.io/badge/work_in-progress-eb3434?style=for-the-badge&labelColor=7d1616)]()\n[![img](https://img.shields.io/badge/license-mit-blueviolet?style=for-the-badge)]()\n\n----\n\n[![forthebadge](https://forthebadge.com/images/badges/built-with-love.svg)](https://forthebadge.com)\n[![forthebadge](https://forthebadge.com/images/badges/uses-git.svg)](https://forthebadge.com)\n[![forthebadge](https://forthebadge.com/images/badges/built-by-hipsters.svg)](https://forthebadge.com)\n\n\n[[_TOC_]]\n\n# Introduction \n\nDdot is a devinstaller application created using the devinstaller framework.\n\n[For more info read below](#what-is-devinstaller)\n\n# Getting Started\n\n```sh\npipx run ddot run\n```\n\nThere needs to be a devfile in the current directory. Any one of these:\n\n- `devfile.toml`\n- `devfile.yaml` or `devfile.yml`\n- `devfile.json`\n\n# Installation\n\nThere are two method to use the application:\n1. Using Pipx to directly run the application \n2. Installing the application on your machine\n\nUsing [Pipx](https://github.com/pipxproject/pipx) method is recommended.\n\n## Method 1: Using Pipx\n\nWhy this method is recommended?\n\nYou can use Pipx to directly run the latest version of ddot without installing it in your machine. This way\nyour machine stays clean and you don\'t need to worry about updating ddot.\n\nYou can also install ddot in your machine using pipx if that\'s what you want.\n\n### Usage without installation\n\n1. Install pipx\n\n- On MacOS\n```sh\nbrew install pipx\npipx ensurepath\n```\n-  Other OS\n\n```sh\npython3 -m pip install --user pipx\npython3 -m pipx ensurepath\n```\n\nfor more information check out [Pipx](https://github.com/pipxproject/pipx).\n\n2. Using ddot\n\n```sh\npipx run ddot <command> # Example: pipx run ddot run --verbose\n```\n\nPlease note that the commands in the section below will need the `pipx run ` prefix with this method. Other method\ndoesn\'t require this prefix.\n\n### Usage with installation\n\n```sh\npipx install ddot\n```\n\n## Method 2: Machine installation\n\nYou can also install ddot without pipx. Although this is discouraged as ddot may disturb your system packages.\n\n```sh\npython -m pip install ddot\n```\n\n# Commands\n\n- Show the ddot version\n```sh\nddot --version\n```\n\n- Show the help menu\n```sh\nddot --help\n```\n\n- Show all the modules available in the devfile\n```sh\nddot show\n```\n\n- Execute the modules\n\n```sh\nddot run\n```\n\nThis command will open up a interactive prompt which you can use to navigate and select all the modules you want to execute.\n\nYou select modules using your SPACEBAR key and naviage using your ARROW keys.\n\n- You can also skip the interactive prompt by running\n\n```sh\nddot run -m <module name>\n```\n\nHere `<module name>` is the name of the module in the devfile at the current directory.\n\n- You can also specify the location to the devfile if it\'s not present in the current directory or if it\'s named differently\n\n```sh\nddot run --spec-file <spec file location>\n```\n\n- You can also set the verbose flag using\n\n```sh\nddot run --verbose\n```\n\nBy default the verbose mode is disabled, so you won\'t see any of the commands that are being executed but instead will see a \nprogress bar.\n\nBut if you need to see it just run using the verbose flag.\n\n# What is Devinstaller?\n\nDevinstaller is a framework to execute the devfiles.\n\n# What are devfiles?\n\nDevfiles is made up of 2 file:\n1. Spec file\n2. Prog file\n\n## Spec file?\n\nThe guiding principle behind the Spec file is to give a declarative and a platform agnostic way to specify a task which then can be handled by the \ndevinstaller application and make that happen.\n\nIn Devinstaller the basic building block for anything is called a "module".\n\nAnd you execute/install the module to do something.\n\n### Modules\n\nWe have modules for:\n1. file\n2. folder\n3. link\n4. app\n5. phony\n6. group\n\n    - `file`, `folder` and `link` modules\n        Say you want to create a file in a declarative way. For this you use the file module\n        and specify where and what the file is and the devinstaller will handle how to create/update/delete\n        the file.\n\n        This way your file module is declarative and the application handles everything required to make it happen, \n        including on how to do it on different OS.\n\n        This is how the `file`, `folder` and the `link` module works.\n\n    - `app` module\n        Then we have the `app` module. App module is for system applications. Here you need to write in some imperative instructions on how\n        a specific application is to be installed. But the beauty behind this you can add in instructions on how to do the same on different\n        platforms and the overall module becomes declarative for other users.\n\n        Example: You can write up the instructions to install ddot on different OS in a devfile, which an another user can import the devfile into\n        their devfile and it will give them a declarative way to install ddot on their machine and they don\'t have to worry about how it\'s going to\n        be done.\n\n    - `phony` and `group` module\n        But we understand that not everything can be put in a declarative way so for those cases we have the `group` and the\n        `phony` module. You can combine these modules in any fashion to translate your imperative instructions into a more\n        declarative way.\n\n### dGenerate\nFor writing up spec file you can use the web application [dGenerate](https://dgenerate.aziraz.com/) or you can write it in any text editor.\n\nUsing dGenerate will give you better user experience. \n\n- Features of dGenerate \n\n    1. Write and save in any file format.\n        Formats supported include:\n        - TOML\n        - YAML\n        - JSON\n\n    2. You can upload existing devfile and do some changes and download it.\n    3. You can share the link to your dGenerate devfile and send it to anyone. And they can go to the link make their changes and download it.\n    4. dGenerate is a completely offline application and it doesn\'t send any data to any servers.\n        So you can be assured of the safety of your data. Even for generating the share link for your devfile, we don\'t store any data on any server.\n        All the data required are in the link itself. You might have probably noticed that all the share links are unusually long and now\n        you know the reason for it.\n\n## Prog file?\n\nTODO\n\n# License\n\nMIT License\n\nCopyright (c) 2021 Justine Kizhakkinedath\n\n',
    'author': 'Justine Kizhakkinedath',
    'author_email': 'justine@kizhak.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://justine.kizhak.com/projects/ddot',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
